library(frm)
library(frmpd)

mcarlo <- function(nrep,N,TT,b=1,ff="logit",PIs,VARs.avZe,design,GMMww.cor=F,bootstrap=F,round=F,missing=F,zeros=1)
{
##### 1. Auxiliary functions

	gen.yX <- function(N,TT,ff,b,PIs,VARs.avZe,round,missing,zeros)
	{
		ai <- rnorm(N,-0.5*VARs.avZe[1],VARs.avZe[1]^.5)
		ai.t <- rep(ai,each=TT)
		vit <- rnorm(N*(TT+1),-0.5*VARs.avZe[2],VARs.avZe[2]^.5)
		vit1 <- vit[-seq(TT+1,N*(TT+1),TT+1)]
		vit <- vit[-seq(1,(N-1)*(TT+1)+1,TT+1)]
		zit <- cbind(rnorm(N*TT,0,VARs.avZe[3]^.5))
		eit <- rnorm(N*TT,0,VARs.avZe[4]^.5)

		xit <- cbind(ai.t,vit1,vit,zit)%*%PIs+eit

		X <- cbind(xit)
		XB <- as.vector(X%*%b)+ai.t+vit

		y <- make.link(ff)$linkinv(XB)
		if(round==T)
		{
			y <- round(y,3)
			y[y==1] <- 0.999
		}
		if(missing==T)
		{
			miss <- rbinom(N*TT,1,zeros)
			y[miss==1] <- 0
		}

		if(PIs[3]==0) return(list(y=y,X=X,ai=ai))
		if(PIs[3]!=0) return(list(y=y,X=X,ai=ai,Z=zit))
	}

	results.save <- function(i,cc,type,results,p.mat,coverage,rej,b,nbetas)
	{
		a <- 1

		p <- results$p
		p.mat[i,((cc-1)*nbetas+1):(cc*nbetas)] <- p[a:(nbetas+a-1)]

		p.var <- results$p.var
		sd <- diag(p.var)^0.5

		if(!is.na(sd[a]))
		{
			coverage[i,cc] <- ((b[1] > p[a]-sd[a]*qnorm(0.975)) & (b[1] < p[a]+sd[a]*qnorm(0.975)))
			rej[i,cc] <- (abs(p[a]/sd[a]) > qnorm(0.975))
		}

		return(list(p.mat=p.mat,coverage=coverage,rej=rej))
	}

##### 2. Preliminary settings 

	nbetas <- length(b)

	est.names <- c("GMMww","GMMc","GMMbgw","GMMpfe","QMLp","QMLpfe")
	est.n <- length(est.names)

	id <- rep(1:N,each=TT)
	time <- rep(1:TT,times=N)

	D <- matrix(0,nrow=N*TT,ncol=N)
	for(j in 1:N) D[((j-1)*TT+1):(j*TT),j] <- 1
	dimnames(D) <- list(NULL,paste("D",1:N,sep=""))

##### 3. Creating matrices to store the results 

	p.mat <- matrix(nrow=nrep,ncol=est.n*nbetas)
	namcol.p <- paste(rep(paste("b",1:nbetas,sep=""),est.n),rep(est.names,each=nbetas),sep=".")
	dimnames(p.mat) <- list(NULL,namcol.p)

	rej <- matrix(NA,nrow=nrep,ncol=est.n)
	coverage <- matrix(NA,nrow=nrep,ncol=est.n)

##### 4. Simulation

	set.seed(111)

	for(i in 1:nrep)
	{
	  ### a) Data generation

		if(i!=1) .Random.seed <<- seed.save

		results <- gen.yX(N,TT,ff,b,PIs,VARs.avZe,round,missing,zeros)
		y <- results$y
		X <- results$X
		ai <- results$ai

		seed.save <- .Random.seed

		dimnames(X) <- list(NULL,"X")

		if(PIs[3]!=0)
		{
			Z <- results$Z
			dimnames(Z) <- list(NULL,"Z")
			x.exogenous <- F
		}
		else x.exogenous <- T

	  ### b) Estimation of structural parameters

	   ## b1) GMMww

		type <- "GMMww"

		cc <- 1
		if(x.exogenous==T) results <- frmpd(id,time,y,X,x.exogenous=x.exogenous,lags=T,start=b,type=type,link=ff,table=F,GMMww.cor=GMMww.cor,bootstrap=bootstrap)
		else results <- frmpd(id,time,y,X,Z,x.exogenous=x.exogenous,lags=T,start=b,type=type,link=ff,table=F,GMMww.cor=GMMww.cor,bootstrap=bootstrap)
		if(results$converged==T)
		{
			results <- results.save(i,cc,type,results,p.mat,coverage,rej,b,nbetas)
			p.mat <- results$p.mat
			coverage <- results$coverage
			rej <- results$rej
		}

	   ## b2) GMMc

		type <- "GMMc"

		cc <- 2
		if(x.exogenous==T) results <- frmpd(id,time,y,X,x.exogenous=x.exogenous,lags=T,start=b,type=type,link=ff,table=F,bootstrap=bootstrap)
		else results <- frmpd(id,time,y,X,Z,x.exogenous=x.exogenous,lags=T,start=b,type=type,link=ff,table=F,bootstrap=bootstrap)
		if(results$converged==T)
		{
			results <- results.save(i,cc,type,results,p.mat,coverage,rej,b,nbetas)
			p.mat <- results$p.mat
			coverage <- results$coverage
			rej <- results$rej
		}

	   ## b3) GMMbgw

		type <- "GMMbgw"

		cc <- 3
		if(x.exogenous==T) results <- frmpd(id,time,y,X,x.exogenous=x.exogenous,lags=F,start=b,type=type,link=ff,table=F,bootstrap=bootstrap)
		else results <- frmpd(id,time,y,X,Z,x.exogenous=x.exogenous,lags=F,start=b,type=type,link=ff,table=F,bootstrap=bootstrap)
		if(results$converged==T)
		{
			results <- results.save(i,cc,type,results,p.mat,coverage,rej,b,nbetas)
			p.mat <- results$p.mat
			coverage <- results$coverage
			rej <- results$rej
		}

	   ## b5) GMMpfe

		type <- "GMMpfe"

		cc <- 4
		if(!any(y==0)) start <- b
		else start <- c(b,ai)
		if(x.exogenous==T) results <- frmpd(id,time,y,X,x.exogenous=x.exogenous,lags=F,start=start,type=type,link=ff,intercept=F,table=F,bootstrap=bootstrap)
		else results <- frmpd(id,time,y,X,Z,x.exogenous=x.exogenous,lags=F,start=start,type=type,link=ff,intercept=F,table=F,bootstrap=bootstrap)
		if(results$converged==T)
		{
			results <- results.save(i,cc,type,results,p.mat,coverage,rej,b,nbetas)
			p.mat <- results$p.mat
			coverage <- results$coverage
			rej <- results$rej
		}

	   ## b7) QMLp

		type <- "QMLp"

		cc <- 5
		results <- frm(y,X,linkfrac=ff,var.type="cluster",var.cluster=id,intercept=F,table=F)
		if(results$converged==T)
		{
			results <- results.save(i,cc,type,results,p.mat,coverage,rej,b,nbetas)
			p.mat <- results$p.mat
			coverage <- results$coverage
			rej <- results$rej
		}

	   ## b8) QMLpfe

		type <- "QMLpfe"

		cc <- 6
		results <- frm(y,cbind(X,D),linkfrac=ff,var.type="cluster",var.cluster=id,intercept=F,table=F)
		if(results$converged==T)
		{
			results <- results.save(i,cc,type,results,p.mat,coverage,rej,b,nbetas)
			p.mat <- results$p.mat
			coverage <- results$coverage
			rej <- results$rej
		}
	}

##### 5. Summary statistics

### a) Parameters

	namrow <- c("N.eff.repl.","Mean","Median","Min","Max","St.Error","RMSE","MAE","Repl.Infer.","rej.b=0","Coverage95")

	for(col in 1:est.n)
	{
		bt <- b

		pp <- as.matrix(p.mat[,col])
		namcol.pp <- namcol.p[col]

		real <- apply(is.na(pp),1,sum)==0
		pp <- as.matrix(pp[real,])
		st.pp <- nrow(pp)
		st.pp <- rbind(st.pp,apply(pp,2,mean))
		st.pp <- rbind(st.pp,apply(pp,2,median))
		st.pp <- rbind(st.pp,apply(pp,2,min))
		st.pp <- rbind(st.pp,apply(pp,2,max))
		st.pp <- rbind(st.pp,sqrt(apply(pp,2,var)))
		st.pp <- rbind(st.pp,sqrt((st.pp[2,]-bt)^2+st.pp[6,]^2))
		st.pp <- rbind(st.pp,apply(abs(pp-matrix(bt,nrow=nrow(pp),ncol=ncol(pp),byrow=T)),2,median))

		real2 <- !is.na(rej[,col])
		st.pp <- rbind(st.pp,sum(real2))
		st.pp <- rbind(st.pp,sum(rej[real2,col])/sum(real2))
		st.pp <- rbind(st.pp,sum(coverage[real2,col])/sum(real2))

		if(col==1)
		{
			st.ppp <- st.pp
			namcol.ppp <- namcol.pp
		}
		else
		{
			st.ppp <- cbind(st.ppp,st.pp)
			namcol.ppp <- c(namcol.ppp,namcol.pp)
		}

		dimnames(st.ppp) <- list(namrow,namcol.ppp)
	}

##### 6. Saving results

	model <- paste("FRM",design,sep="-")
	model <- paste(model,ff,sep="-G")
	model <- paste(model,N,sep="-N")
	model <- paste(model,TT,sep="-T")
	if(bootstrap==T) model <- paste(model,"boot",sep="-")

	filename <- paste(model,"dat",sep=".")
	write.table(round(p.mat,8),file=filename,sep="\t",row.names=F,quote=F)

	filename <- paste(model,"btcov",sep=".")
	write.table(round(st.ppp,5),file=filename,sep="\t",quote=F)

##### 7. Printing results

	cat("\n####################   MODEL",model,"  ####################\n")

	cat("\nEXPERIMENTAL DESIGN\n")
	cat("\n Number of replications:",nrep)
	cat("\n Design:",design)
	cat("\n Link function:",ff)
	cat("\n Number of cross-sectional units:",N)
	cat("\n Number of time periods:",TT)
	cat("\n Structural parameters:",b)
	cat("\n Reduced form parameters:",PIs)
	cat("\n VARs.avZe:",VARs.avZe)

	cat("\n")
	cat("\nRESULTS\n")
	cat("\n")
	print(round(st.ppp,3))
	cat("\n")
}

##### ##### ##### ##### ##### D. RUNS ##### ##### ##### ##### #####

nrep <- 5000

##### Figure 1

EXP1 <- c(-1,-0.5,-0.25,0,0.25,0.5,1)
for(j in EXP1) mcarlo(nrep,N=50,TT=5,PIs=c(j,0,0,0),VARs.avZe=c(0.5,1,0.5,0.5),design=paste("EXP1",j,sep="_"))

EXP2 <- c(0.05,0.5,1,1.5,2,2.5,3)
for(j in EXP2) mcarlo(nrep,N=50,TT=5,PIs=c(0.5,0,0,0),VARs.avZe=c(0.5,j,0.5,0.5),design=paste("EXP2",j,sep="_"))

EXP3 <- c(0,0.15,0.3,0.45,0.6,0.8,1)
for(j in EXP3) mcarlo(nrep,N=50,TT=5,PIs=c(0.5,j,0,0),VARs.avZe=c(0.5,1,0.5,0.5),design=paste("EXP3",j,sep="_"))

EXP4 <- seq(0,1.5,0.25)
for(j in EXP4) mcarlo(nrep,N=50,TT=5,PIs=c(0.5,0,j,1),VARs.avZe=c(0.5,1,0.5,0.5),design=paste("EXP4",j,sep="_"))


##### Figure 2

TT.set <- c(2,3,10,15,25,30,40,100)
for(j in TT.set) mcarlo(nrep,N=50,TT=j,PIs=c(0.5,1,0,0),VARs.avZe=c(0.5,1,0.5,0.5),design="EXP3_1",GMMww.cor=T)


##### Figure 3

EXP5 <- seq(0,0.24,0.04)
for(j in EXP5) mcarlo(nrep,N=50,TT=5,PIs=c(0.5,0,0,0),VARs.avZe=c(0.5,1,0.5,0.5),design=paste("EXP5",j,sep="_"),missing=T,zeros=j)

EXP6 <- seq(0.5,3,0.5)
for(j in EXP6) mcarlo(nrep,N=50,TT=5,PIs=c(0.5,0,0,0),VARs.avZe=c(j,1,0.5,0.5),design=paste("EXP6",j,sep="_"),round=T)

rm(list=ls())

